try:
    from django import newforms as forms
    from django.newforms.forms import BoundField
except ImportError:
    from django import forms
    from django.forms.forms import BoundField

from django.utils.html import escape
from django.utils.encoding import force_unicode
from django.utils.safestring import mark_safe
from django.template.defaultfilters import slugify

class SectionedForm(object):
    fieldsets = ()
    fieldset_template = '<h3 id="section_%(section)s">%(title)s</h3>'

    def _render_fieldset(self, fieldset, fields, normal_row, help_text_html, errors_on_separate_row):
        output, hidden_fields = [], []

        if fieldset:
            output.append(normal_row%{
                'errors': '',
                'label': '&nbsp;',
                'field': self.fieldset_template%{
                    'title': fieldset,
                    'section': slugify(fieldset),
                    },
                'help_text': '',
                })

        #for name, field in [i for i in self.fields.items() if i[0] in fields]:
        for name in fields:
            field = filter(lambda f: f[0] == name, self.fields.items())[0][1]

            bf = BoundField(self, field, name)
            bf_errors = self.error_class([escape(error) for error in bf.errors]) # Escape and cache in local variable.
            if bf.is_hidden:
                if bf_errors:
                    top_errors.extend([u'(Hidden field %s) %s' % (name, force_unicode(e)) for e in bf_errors])
                hidden_fields.append(unicode(bf))
            else:
                if errors_on_separate_row and bf_errors:
                    output.append(error_row % force_unicode(bf_errors))
                if bf.label:
                    label = escape(force_unicode(bf.label))
                    # Only add the suffix if the label does not end in
                    # punctuation.
                    if self.label_suffix:
                        if label[-1] not in ':?.!':
                            label += self.label_suffix
                    label = bf.label_tag(label) or ''
                else:
                    label = ''
                if field.help_text:
                    help_text = help_text_html % force_unicode(field.help_text)
                else:
                    help_text = u''
                output.append(normal_row % {
                    'errors': force_unicode(bf_errors),
                    'label': force_unicode(label),
                    'field': unicode(bf),
                    'help_text': help_text,
                    'field_name': unicode(name),
                    })

        return output, hidden_fields

    @property
    def fieldsets_html(self):
        normal_row = u'<tr><th>%(label)s</th><td>%(errors)s%(field)s%(help_text)s</td></tr>'
        error_row = u'<tr><td colspan="2">%s</td></tr>'
        row_ender = '</td></tr>'
        help_text_html = u'<br />%s'
        errors_on_separate_row = False
        output = []

        if not hasattr(self, '_fieldsets_and_hidden_fields'):
            r_fields = [self._render_fieldset(fieldset, fields, normal_row, help_text_html,
                errors_on_separate_row) for fieldset, fields in self.fieldsets]
            r_fieldsets = [mark_safe('\n'.join(f[0])) for f in r_fields]
            r_hidden = mark_safe('\n'.join(reduce(lambda a,b: a + b, [f[1] for f in r_fields])))

            self._fieldsets_and_hidden_fields = r_fieldsets + [r_hidden]

        output = self._fieldsets_and_hidden_fields

        return output

    def _html_output(self, normal_row, error_row, row_ender, help_text_html, errors_on_separate_row):
        "Helper function for outputting HTML. Used by as_table(), as_ul(), as_p()."
        top_errors = self.non_field_errors() # Errors that should be displayed above all fields.
        output, hidden_fields = [], []

        for fieldset, fields in self.fieldsets:
            f_output, f_hidden_fields = self._render_fieldset(fieldset, fields, normal_row,
                    help_text_html, errors_on_separate_row)
            output += f_output
            hidden_fields += f_hidden_fields

        if top_errors:
            output.insert(0, error_row % force_unicode(top_errors))
        if hidden_fields: # Insert any hidden fields in the last row.
            str_hidden = u''.join(hidden_fields)
            if output:
                last_row = output[-1]
                # Chop off the trailing row_ender (e.g. '</td></tr>') and
                # insert the hidden fields.
                output[-1] = last_row[:-len(row_ender)] + str_hidden + row_ender
            else:
                # If there aren't any rows in the output, just append the
                # hidden fields.
                output.append(str_hidden)
        return mark_safe(u'\n'.join(output))





VERSION = (1, 1, 37, 'stable')

def get_version():
    return '%d.%d.%d-%s'%VERSION

__author__ = 'Marinho Brandao'
#__date__ = '$Date: 2008-07-26 14:04:51 -0300 (Ter, 26 Fev 2008) $'[7:-2]
__license__ = 'GNU Lesser General Public License (LGPL)'
__url__ = 'http://django-plus.googlecode.com'
__version__ = get_version()

def get_dynamic_template(slug, context=None):
    from models import DynamicTemplate

    return DynamicTemplate.objects.get(slug=slug).render(context or {})




# Admin registrations
from django.contrib import admin
from django.contrib.admin.options import ModelAdmin

from models import DynamicTemplate, StaticFile

class AdminDynamicTemplate(ModelAdmin):
    list_display = ('title','slug','group',)
    search_fields = ('title','content','group')

admin.site.register(DynamicTemplate, AdminDynamicTemplate)
admin.site.register(StaticFile)




from optparse import make_option

from django.core.management import BaseCommand
from django.conf import settings
from django.db import connection, transaction, models

def get_auto_fields(model):
    return [f for f in model._meta.fields if isinstance(f, models.AutoField)]

SQL_RESET_TPL = "select setval('%(table)s_%(field)s_seq', (SELECT max(%(field)s) + 1 from %(table)s));"
#SQL_RESET_TPL = "select setval('%(table)s_%(field)s_seq', 1);"

class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
            make_option('--execute',
                default=False,
                dest='execute',
                action='store_true',
                ),
            )

    def handle(self, execute=False, **kwargs):
        print 'Checking...'

        if execute:
            cur = connection.cursor()

        # All applications
        commands = []
        for app in models.get_apps():
            app_name = app.__name__.split('.')[-2]
            model_list = models.get_models(app)
            
            for model in model_list:
                fields = get_auto_fields(model)
                if not fields:
                    continue

                for field in fields:
                    sql = SQL_RESET_TPL%{
                            'table': model._meta.db_table, 
                            'field': field.db_column or field.name,
                            }
                    print sql
                    
                    if execute:
                        cur.execute(sql)




def path_to_object(path):
    """Returns a Python object from a string path"""
    dot = '.' in path and path.rindex('.') or None
    
    if dot:
        f_mod, f_obj = path[:dot], path[dot+1:]

        mod = __import__(f_mod, {}, {}, [''])
        obj = getattr(mod, f_obj)
    else:
        obj = __import__(path, {}, {}, [''])

    return obj

def split1000(s, sep=','):
    """http://www.python.org.br/wiki/FormatarNumeros"""
    #return s if len(s) <= 3 else split1000(s[:-3], sep) + sep + s[-3:]
    return len(s) <= 3 and s or split1000(s[:-3], sep) + sep + s[-3:]

def get_admin_url(obj):
    from django.contrib import admin
    admin_root_path = admin.site.root_path

    return '%s%s/%s/%s/'%(
            admin_root_path,
            obj._meta.app_label,
            obj.__class__.__name__.lower(),
            obj.pk,
            )

def get_full_url(url):
    from django.contrib.sites.models import Site
    site = Site.objects.get_current()

    url = url.startswith('/') and url or '/' + url

    return 'http://%s%s'%(site.domain, url)




from django.conf import settings
from django.contrib.sites.models import Site

MONETARY_LOCALE = getattr(settings, 'MONETARY_LOCALE', '')
THOUSANDS_SEPARATOR = getattr(settings, 'THOUSANDS_SEPARATOR', '')
STATIC_FILES_PATH = getattr(settings, 'STATIC_FILES_PATH', 'uploads')

ROBOT_PROTECTION_DOMAIN = getattr(settings, 'ROBOT_PROTECTION_DOMAIN', None)

RESULT_OK = getattr(settings, 'RESULT_OK', 'ok')
RESULT_ERROR = getattr(settings, 'RESULT_ERROR', 'error')




"""Utilities for test modules"""
import os, unittest, doctest
from sets import Set

from django.core.serializers import deserialize
from django.db.models import get_apps
from django.test.simple import get_tests, run_tests as old_run_tests

def load_fixture(path, file_type='json'):
    """Load a fixture file"""
    fp = file(path)
    cont = fp.read()
    fp.close()

    for obj in deserialize(file_type, cont):
        obj.save()

def model_has_fields(model_class, fields):
    """Checks if a model class has all fields in fields list and returns a
    list of fields that aren't in one of them.
    
    This method returns an empty list ( [] ) when everything is ok"""
    fields = Set(fields)
    model_fields = Set([f.name for f in model_class._meta.fields])
    return list(fields - model_fields)

def is_model_class_fk(model_class_from, field, model_class_to):
    """Returns True if field is ForeignKey to model class informed"""
    return issubclass(
            model_class_from._meta.get_field_by_name(field)[0].rel.to,
            model_class_to,
            )

def is_field_type(model_class_from, field, field_type, **kwargs):
    """Checks if a field of a model class if of the type informed.
    If field_type value is a class, it compares just the class of field,
    if field_type is an instance of a field type class, it compares the
    max_length, max_digits and decimal_places, blank and null"""
    field = model_class_from._meta.get_field_by_name(field)[0]

    if field.__class__ != field_type:
        return False

    for k,v in kwargs.items():
        if kwargs[k] != getattr(field, k, None):
            raise Exception('%s: %s'%(k, unicode(getattr(field, k, None))))
            return False

    return True

def is_model_pk(model_class, field):
    """Checks if a field is the primary key of the model class"""
    return model_class._meta.pk.name == field

def run_tests(test_labels, verbosity=1, interactive=True, extra_tests=[]):
    """Test runner to support many DocTests *.txt files and TestUnits *.py
    using a setting TEST_FILES in app.tests module"""
    for app in get_apps():
        test_mod = get_tests(app)

        if not test_mod or hasattr(test_mod, 'suite'):
            continue

        suites = []

        # DocTest files
        for filename in getattr(test_mod, 'DOCTEST_FILES', []):
            try:
                suites.append(doctest.DocFileSuite(
                    filename,
                    package=test_mod,
                    encoding='utf-8',
                    ))
            except TypeError:
                suites.append(doctest.DocFileSuite(
                    filename,
                    package=test_mod,
                    ))

        # Unit Tests modules
        for module in getattr(test_mod, 'UNITTEST_MODULES', []):
            suites.append(unittest.TestLoader().loadTestsFromModule(module))

        # Sets the 'suites' attribute to test module
        if suites:
            print suites
            test_mod.suite = lambda: unittest.TestSuite(suites)

    return old_run_tests(test_labels, verbosity, interactive, extra_tests)

def url_status_code(url, status_code=200, content=None, client=None, return_response=False):
    """Checks if the informed URL returns the wanted status_code"""
    if not client:
        from django.test.client import Client
        client = Client()

    resp = client.get(url)

    if return_response:
        return resp

    ret = True

    if status_code and status_code == resp.status_code:
        ret = ret and True

    if content and content == resp.status_code:
        ret = ret and True

    return ret




# -*- coding: utf-8 -*-
from django.forms import widgets
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext as _
from django.utils.datastructures import MultiValueDict
from django.http import HttpResponse
from django.forms.util import flatatt
from django.utils.encoding import force_unicode
from django.conf import settings
from django.views.decorators.cache import never_cache

# id="ajaxmultiselect_field_%(name)s"
# style="padding: 5px; margin-left: 105px;"
TPL_UL = """
<ul class="ajaxmultiselect_field" %(attrs)s>
    %(items)s
</ul>
"""

TPL_LI = """
<li id="li_%(name)s_%(id)d">
    %(repr)s <img src="/admin_media/img/admin/icon_deletelink.gif" class="ajaxmultiselect_remove"/>
    <input type="hidden" name="%(name)s" value="%(id)d"/>
</li>
"""

TPL_LI_ADD = """
<li id="li_%(name)s_add">
    <input id="ajaxmultiselect_input_%(name)s_add" autocomplete="off" class="ajaxmultiselect_add"/>
    <script type="text/javascript" defer="defer">
        AUTO_TEMP_ID_%(name)s = '';

        function onSelect(data) {
            AUTO_TEMP_ID_%(name)s = data.id;
        }

        $('input#ajaxmultiselect_input_%(name)s_add').autocomplete(
            "%(url)s",
            {
                "autoFill": true,
                "multiple": false,
                "select": onSelect,
                "minChars": 0,
                "scroll": false
            }
        );
    </script>

    <img src="/admin_media/img/admin/icon_addlink.gif" class="ajaxmultiselect_add"/>
</li>
"""

TPL_SCRIPT = """
<script type="text/javascript" defer="defer">
    $('ul#id_%(name)s>li>img.ajaxmultiselect_add').click(function(){
        if (!AUTO_TEMP_ID_%(name)s) {
            alert('You must choose a valid item!');
            return
        } else if (document.getElementById("li_%(name)s_"+AUTO_TEMP_ID_%(name)s)) {
            alert('You already added this!');
            return
        }

        var li = document.createElement('li');
        li.id = "li_%(name)s_"+AUTO_TEMP_ID_%(name)s;

        var txt = document.createTextNode($('#ajaxmultiselect_input_%(name)s_add').val());
        li.appendChild(txt);

        var img = document.createElement('img');
        img.src = "/admin_media/img/admin/icon_deletelink.gif";
        img.className = "ajaxmultiselect_remove";
        li.appendChild(img);

        var inp = document.createElement('input');
        inp.type = "hidden";
        inp.name = "%(name)s"
        inp.value = AUTO_TEMP_ID_%(name)s;
        li.appendChild(inp);

        document.getElementById('id_%(name)s').insertBefore(li, document.getElementById('li_%(name)s_add'))

        $('ul#id_%(name)s>li>img.ajaxmultiselect_remove').click(ajaxMultiSelectRemove);
    });

    function ajaxMultiSelectRemove() {
        $(this).parent().remove();
    }

    $('ul#id_%(name)s>li>img.ajaxmultiselect_remove').click(ajaxMultiSelectRemove);
</script>
"""

class AjaxMultiSelect(widgets.Widget):
    model = None
    auto_url = None

    class Media:
        js = (settings.MEDIA_URL+'js/jquery-packed.js',
              settings.MEDIA_URL+'js/jquery.autocomplete.js',
              settings.MEDIA_URL+'js/jquery.bgiframe.min.js',)
        css = {'all':
                (settings.MEDIA_URL+'css/jquery.autocomplete.css',
                 settings.MEDIA_URL+'css/ajax_m2m_widget.css',)
                }

    def __init__(self, verbose_name, model, auto_url, attrs=None):
        self.verbose_name = verbose_name
        self.model = model
        self.auto_url = auto_url

        super(AjaxMultiSelect, self).__init__(attrs)

    def build_attrs(self, extra_attrs=None, **kwargs):
        ret = super(AjaxMultiSelect, self).build_attrs(extra_attrs=None, **kwargs)

        if 'style' in ret:
            ret['style'] += 'padding: 5px; '
        else:
            ret['style'] = 'padding: 5px; '

        return ret

    def render(self, name, value, attrs=None, choices=()):
        final_attrs = self.build_attrs(attrs)
        final_attrs.setdefault('id', 'id_'+name)

        if value:
            items = []
            
            for id in value:
                try:
                    items.append(self.model.objects.get(id=id))
                except self.model.DoesNotExist, e:
                    pass
        else:
            items = []
        
        li_items = '\n'.join([TPL_LI %{'name': name, 'id': item.id, 'repr': unicode(item)} for item in items])
        li_add = TPL_LI_ADD %{'name': name, 'url': self.auto_url}

        output = []
        output.append(TPL_UL %{'name': name , 'items': '\n'.join((li_items, li_add)), 'attrs': flatatt(final_attrs)})
        output.append(TPL_SCRIPT %{'name': name, 'url': self.auto_url})

        return mark_safe(u'\n'.join(output))

    def value_from_datadict(self, data, files, name):
        if isinstance(data, MultiValueDict):
            return data.getlist(name)
        return data.get(name, None)

@never_cache
def auto_complete_view(request, query_func, desc_field, id_field, limit=15, show_empty=True, extra_args=False):
    def iter_results(results):
        if results:
            for r in results:
                #yield '%s|%s\n' % (desc_field and getattr(r, desc_field) or unicode(r), getattr(r, id_field))
                yield unicode('%s|%s\n' % (desc_field and getattr(r, desc_field) or unicode(r), getattr(r, id_field)))
    
    if not show_empty and not request.GET.get('q'):
        return HttpResponse(mimetype='text/plain')
    
    q = request.GET.get('q', '')

    if extra_args:
        args = dict(request.GET.items())
        items = query_func(q, args)[:limit]
    else:
        items = query_func(q)[:limit]

    res = iter_results(items)

    return HttpResponse([x for x in res], mimetype='text/plain')




# -*- coding: utf-8 -*-
import operator

from django.forms.widgets import TextInput
from django.conf import settings
from django.utils.safestring import mark_safe
from django.utils import simplejson
from django.core.urlresolvers import reverse
from django.db import models
from django.http import HttpResponse
from django.contrib import admin
from django.shortcuts import render_to_response
from django.template import RequestContext

from djangoplus.utils import path_to_object, get_admin_url
from djangoplus import app_settings

registered_models = {}

class AjaxFKWidget(TextInput):
    class Media:
        js = (settings.MEDIA_URL+'js/ajax_fk_widget.js',)
        css = {'all': (settings.MEDIA_URL+'css/ajax_fk_widget.css',)}

    fill_left_zeros = 0
    rel = None
    model = None
    related_field = None
    filter_dict = {}

    window_url = None
    load_url = None
    add_url = None

    def __init__(self, *args, **kwargs):
        if 'rel' in kwargs:
            self.rel = kwargs.pop('rel')
            self.get_from_rel()
        else:
            self.model = kwargs.pop('model', None)
            self.related_field = kwargs.pop('related_field', 'pk')

        if 'fill_left_zeros' in kwargs:
            self.fill_left_zeros = kwargs.pop('fill_left_zeros')

        self.window_url = kwargs.pop('window_url', self.window_url)
        if self.window_url is None:
            self.window_url = reverse('ajax-fk-window-url', kwargs={
                'app': self.model.__module__.split('.')[0],
                'model': self.model.__name__,
                })

        self.load_url = kwargs.pop('load_url', self.load_url)
        if self.load_url is None:
            self.load_url = reverse('ajax-fk-load-url', kwargs={
                'app': self.model.__module__.split('.')[0],
                'model': self.model.__name__,
                })

        self.add_url = kwargs.pop('add_url', self.add_url)
        if self.add_url is None:
            self.add_url = '%s%s/%s/add/'%(
                    admin.site.root_path,
                    self.model.__module__.split('.')[0],
                    self.model.__name__.lower(),
                    )

        super(AjaxFKWidget, self).__init__(*args, **kwargs)

    def get_from_rel(self):
        self.model = self.rel.to
        self.related_field = self.rel.get_related_field().name

    def value_from_datadict(self, data, files, name):
        ret = super(AjaxFKWidget, self).value_from_datadict(data, files, name)
        
        return ret or None

    def render(self, name, value, attrs=None):
        attrs = attrs or {}
        attrs['class'] = attrs.get('class', '') + ' ajax-fk'

        ret = super(AjaxFKWidget, self).render(name, value, attrs)
        class_path = '.'.join([self.model.__module__, self.model.__name__])

        display = self.make_display(attrs, value)

        script = self.make_script(name)

        return mark_safe(' '.join([ret, display, script]))

    def make_display(self, attrs, value):
        url = self.get_display_url(value)

        return '<a id="%s_display" class="ajax-fk-display" %s>%s</a>'%(
                attrs['id'],
                url and 'href="%s"'%url or '',
                value and self.label_for_value(value) or '(nenhum selecionado)',
                )

    def make_script(self, name):
        sc_params = {
                "model": '.'.join([self.model.__module__,self.model.__name__]),
                "filter": self.filter_dict or {},
                "window-url": self.window_url,
                "load-url": self.load_url,
                "add-url": self.add_url,
                "fill-left-zeros": self.fill_left_zeros,
                "callback-funcion-on-change": '', # JavaScript function name to call after
                                                  # change the widget value. This function
                                                  # must waits for a param to receive the
                                                  # JSON of changed values
                }

        # Loads extra parameters from its driver
        try:
            driver = registered_models[self.model]

            sc_params.update(driver.get_extra_params(self, sc_params))
        except KeyError:
            pass

        script = """<script type="text/javascript">fk_widgets['%s'] = %s;%s</script>"""%(
                name,
                simplejson.dumps(sc_params),
                self.get_additional_script(name)
                )

        return script

    def get_additional_script(self, name):
        """To be used to extend script functions"""
        return ''

    def label_for_value(self, value):
        obj = self.model._default_manager.get(**{self.related_field: value})
        return unicode(obj)

    def get_display_url(self, value):
        global registered_models
        
        try:
            driver = registered_models[self.model]
        except KeyError:
            return ''

        try:
            obj = self.model._default_manager.get(**{self.related_field: value})

            return get_admin_url(obj)
        except self.model.DoesNotExist:
            return ''

    @classmethod
    def register(cls, driver_class):
        """Register a model class with its driver class
        Example: AjaxFKWidget.register(MyModelClass, AjaxFKMyModelClass)"""
        global registered_models

        registered_models[driver_class.model] = driver_class

class AjaxFKDriver(object):
    model = None
    list_display = None
    search_fields = None
    list_filter = None
    ordering = None

    def __init__(self, request, cls=None):
        self.request = request
        self.cls = cls

    def get_columns(self):
        return [self.model._meta.get_field_by_name(f)[0].verbose_name.capitalize() for f in self.list_display]

    def get_results(self):
        qs = self.get_query_set()

        # Keyword search
        qs = self.search_by_fields(qs)

        # Filters
        for k,v in self.request.GET.items():
            if not k.startswith('ajax-fk-filter-'):
                continue

            f_name = k[15:]

            qs = qs.filter(**{str(f_name): v})

        return qs

    def search_by_fields(self, qs):
        # Apply keyword searches.
        def construct_search(field_name):
            if field_name.startswith('^'):
                return "%s__istartswith" % field_name[1:]
            elif field_name.startswith('='):
                return "%s__iexact" % field_name[1:]
            elif field_name.startswith('@'):
                return "%s__search" % field_name[1:]
            else:
                return "%s__icontains" % field_name

        if self.search_fields and self.request.GET.get('ajax_fk_search', ''):
            for bit in self.request.GET['ajax_fk_search'].split():
                or_queries = [models.Q(**{construct_search(str(field_name)): bit}) for field_name in self.search_fields]
                qs = qs.filter(reduce(operator.or_, or_queries))
            for field_name in self.search_fields:
                if '__' in field_name:
                    qs = qs.distinct()
                    break

        return qs

    def get_by_pk(self):
        try:
            obj = self.model.objects.get(pk=self.request.GET['pk'])

            ret = {
                'res': app_settings.RESULT_OK,
                'pk': obj.pk,
                'display': unicode(obj),
                'url': self.get_display_url(obj),
            }
        except self.model.DoesNotExist:
            ret = {
                'res': app_settings.RESULT_ERROR,
                'msg': u'%s não encontrado!'%self.model._meta.verbose_name,
            }

        return ret

    def get_query_set(self):
        qs = self.model.objects.all()

        # Ordering
        if self.ordering:
            qs = qs.order_by(*self.ordering)

        return qs

    @classmethod
    def get_display_url(cls, obj):
        """Returns the url to show as a link on the object display when selected.
        
        The default value is the Admin URL for this object.
        
        You can also inform an empty string to set it as disabled."""
        return get_admin_url(obj)

    @classmethod
    def get_extra_params(cls, widget=None, current_params=None):
        """Returns extra params to append to the parameters dictionary of widgets
        that use this model class."""
        return {}

def window_view(request, app, model):
    cls = path_to_object('%s.models.%s'%(app, model))
    driver = registered_models[cls](request, cls)

    columns = driver.get_columns()
    results = driver.get_results()
    list_display = driver.list_display
    function_get_url = driver.get_display_url

    return render_to_response(
            'djangoplus/ajaxfkwidget_window.html',
            locals(),
            context_instance=RequestContext(request),
            )

def load_view(request, app, model):
    cls = path_to_object('%s.models.%s'%(app, model))
    driver = registered_models[cls](request, cls)

    ret = driver.get_by_pk()

    return HttpResponse(simplejson.dumps(ret), mimetype='text/javascript')




from jquery_auto_complete import JQueryAutoComplete
from ajax_m2m_widget import AjaxMultiSelect, auto_complete_view
from dropdown_multi_widget import DropDownMultiple
from raw_id_widget import ModelRawIdWidget



from django.http import HttpResponseRedirect

class HttpMessageMiddleware(object):
    def process_exception(self, request, exception):
        if request.user.is_authenticated() and exception is HttpMessage:
            request.user.message_set.create(message=unicode(exception))
            return HttpResponseRedirect(request.META.get('HTTP_REFERER', '/'))

class HttpMessage(Exception):
    pass




from django.db import models
from django import template
from django.template.defaultfilters import slugify

import app_settings

class DynamicTemplate(models.Model):
    class Meta:
        ordering = ('title',)

    title = models.CharField(max_length=100)
    slug = models.SlugField(blank=True, unique=True)
    group = models.SlugField(blank=True)
    content = models.TextField()

    def __unicode__(self):
        return self.title

    def render(self, context):
        return template.Template(self.content).render(template.Context(context))

class StaticFile(models.Model):
    file = models.FileField(upload_to=app_settings.STATIC_FILES_PATH)
    label = models.CharField(max_length=50, blank=True)
    group = models.CharField(max_length=50, blank=True)
    mimetype = models.CharField(max_length=50, blank=True)

    def __unicode__(self):
        return self.label or self.file.name

# SIGNALS AND LISTENERS
from django.db.models import signals

# DynamicTemplate
def dynamictemplate_pre_save(sender, instance, signal, *args, **kwargs):
    # Cria slug
    if not instance.slug:
        instance.slug = slugify(instance.title)

    if not instance.group:
        instance.group = slugify(instance.group)

signals.pre_save.connect(dynamictemplate_pre_save, sender=DynamicTemplate)




import urllib

from django.conf import settings
from django.db import models

class AbstractGMapLocalized(models.Model):
    map_latitude = models.FloatField(null=True, blank=True)
    map_longitude = models.FloatField(null=True, blank=True)
    map_manual_localized = models.BooleanField(default=False)

    class Meta:
        abstract = True

    def get_google_maps_url(self):
        return "http://local.google.com/maps?f=q&hl=pt-BR&ll=%s,%s" %(self.map_latitude, self.map_longitude)

    def get_latitude(self):
        return self.map_latitude and "%.6f" % float(self.map_latitude) or None

    def get_longitude(self):
        return self.map_longitude and "%.6f" % float(self.map_longitude) or None

    def localization_default(self):
        """Retorns true if this is in default locazation"""
        return (not self.map_latitude and not self.map_longitude) or \
                (float(self.map_latitude) == settings.GOOGLE_MAPS_LATITUDE_DEFAULT and \
                 float(self.map_longitude) == settings.GOOGLE_MAPS_LONGITUDE_DEFAULT)

    def get_full_address(self):
        return ''

def get_geocode(address):
    """http://www.djangosnippets.org/snippets/293/"""
    try:
        import urllib

        key = settings.GOOGLE_MAPS_API_KEY
        output = "csv"
        location = urllib.quote_plus(address.encode('utf-8'))
        url = "http://maps.google.com/maps/geo?q=%s&output=%s&key=%s" % (location, output, key)
        data = urllib.urlopen(url).read()
        dlist = data.split(',')

        if dlist[0] == '200':
            return dlist[2], dlist[3]
        else:
            return None, None
    except Exception, e:
        return None, None




import hmac, sha, base64, os

from django.conf import settings
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.http import HttpResponseRedirect, HttpResponse
from django.utils.translation import ugettext as _
from django.template.defaultfilters import slugify

try:
    import Image, ImageDraw, ImageFont
except:
    from PIL import Image, ImageDraw, ImageFont

from djangoplus import app_settings

class FakeSessionCookieMiddleware(object):
    """
    Thanks to Dan Fairs
    The source post is available at:
    http://www.stereoplex.com/two-voices/cookieless-django-sessions-and-authentication-without-cookies
    """
    def process_request(self, request):
        if request.GET.has_key(settings.SESSION_COOKIE_NAME):
            request.COOKIES[settings.SESSION_COOKIE_NAME] = request.GET[settings.SESSION_COOKIE_NAME]
        elif request.POST.has_key(settings.SESSION_COOKIE_NAME):
            request.COOKIES[settings.SESSION_COOKIE_NAME] = request.POST[settings.SESSION_COOKIE_NAME]

class ProtectAntiRobotsMiddleware(object):
    def process_request(self, request):
        current_domain = app_settings.ROBOT_PROTECTION_DOMAIN

        if request.path_info.startswith('/protectantirobots/'):
            path = request.GET.get('path', '') or request.POST.get('path', '')
            if request.path == '/protectantirobots/img/':
                s = base64.b64decode(request.COOKIES['protectantirobots_key_'+path])
                size = (100,30)
                img = Image.new("RGB", size, "white")

                draw = ImageDraw.Draw(img)
                font = ImageFont.truetype(os.path.dirname(__file__)+"/FreeSansBold.ttf", 24)
                draw.text((2,2), s, fill="red", font=font)

                draw.line((0, 0) + img.size, fill=128)
                draw.line((0, img.size[1], img.size[0], 0), fill=128)
                del draw

                ret = HttpResponse(mimetype='image/gif')
                img.save(ret, "GIF")
            elif 'k' in request.GET:
                if request.POST:
                    n = request.POST['n']
                    sec = base64.b64decode(request.COOKIES['protectantirobots_key_'+path])
                    if n == sec:
                        ret = HttpResponseRedirect(request.COOKIES['protectantirobots_referer_'+path])
                        ret.set_cookie('protectantirobots_sec_'+str(path), sec, domain=current_domain)
                    else:
                        ret = render_to_response(
                            'djangoplus/protectantirobots.html',
                            {'msg': _('Invalid number!'), 'path': path},
                            context_instance=RequestContext(request),
                        )
                else:
                    ret = render_to_response(
                        'djangoplus/protectantirobots.html',
                        locals(),
                        context_instance=RequestContext(request),
                        )
                    ret.set_cookie('protectantirobots_key_'+str(path), request.REQUEST['k'], domain=current_domain)
                    ret.set_cookie('protectantirobots_referer_'+str(path), request.META.get('HTTP_REFERER', ''), domain=current_domain)

            return ret




from django.http import HttpResponse
from django.utils import simplejson
from django.views.decorators.cache import never_cache

@never_cache
def json_list_view(request, queryset, desc_field, id_field, extra_args=False):
    if request.GET:
        kwargs = dict([(str(k),v) for k,v in request.GET.items()])
        q = queryset.filter(**kwargs)
    else:
        q = queryset.all()

    ret = q.values(id_field, desc_field)

    return HttpResponse(simplejson.dumps([x for x in ret]), mimetype='text/javascript')




# -*- coding: utf-8 -*-
from django.forms import widgets
from django.utils.safestring import mark_safe
from django.utils.datastructures import MultiValueDict
from django.forms.util import flatatt

TPL_OPTION = """<option value="%(value)s" %(selected)s>%(desc)s</option>"""

TPL_SELECT = """
<select class="dropdown_select" %(attrs)s>
%(opts)s
</select>
"""

TPL_SCRIPT = """
<script type="text/javascript" defer="defer">
    $('span#%(id)s>select.dropdown_select').change(function(){
        var pattern = 'span#%(id)s>select.dropdown_select';
        var last_item = $(pattern+':last');

        if (last_item.val()) {
            last_item.clone(true).appendTo($('span#%(id)s'));
            $('span#%(id)s').append(' ');
        };

        var values = [];

        for (var i=$(pattern).length-1; i>=0; i--) {
            if (values.indexOf($($(pattern).get(i)).val()) >= 0) {
                $($(pattern).get(i)).remove();
            } else {
                values.push($($(pattern).get(i)).val());
            }
        };
    });
</script>
"""

TPL_FULL = """
<span class="dropdown_multiple" id="%(id)s">
%(values)s
%(script)s
</span>
"""

class DropDownMultiple(widgets.Widget):
    choices = None

    def __init__(self, attrs=None, choices=()):
        self.choices = choices

        super(DropDownMultiple, self).__init__(attrs)

    def render(self, name, value, attrs=None, choices=()):
        if value is None: value = []
        final_attrs = self.build_attrs(attrs, name=name)

        # Pop id
        id = final_attrs['id']
        del final_attrs['id']

        # Insert blank value
        choices = [('','---')] + list(self.choices)

        # Build values
        items = []
        for val in value:
            opts = "\n".join([TPL_OPTION %{'value': k, 'desc': v, 'selected': val == k and 'selected="selected"' or ''} for k, v in choices])
            
            items.append(TPL_SELECT %{'attrs': flatatt(final_attrs), 'opts': opts})

        # Build blank value
        opts = "\n".join([TPL_OPTION %{'value': k, 'desc': v, 'selected': ''} for k, v in choices])
        items.append(TPL_SELECT %{'attrs': flatatt(final_attrs), 'opts': opts})

        script = TPL_SCRIPT %{'id': id}
        output = TPL_FULL %{'id': id, 'values': '\n'.join(items), 'script': script}

        return mark_safe(output)

    def value_from_datadict(self, data, files, name):
        if isinstance(data, MultiValueDict):
            return [i for i in data.getlist(name) if i]
        
        return data.get(name, None)




import types, re, decimal
from datetime import date, time, datetime

from django.utils.safestring import mark_safe
from django.db import models
from django.template.defaultfilters import yesno, linebreaksbr, urlize
from django.utils.translation import get_date_formats
from django.utils.text import capfirst
from django.utils import dateformat
from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist

from djangoplus.templatetags.djangoplus_tags import moneyformat
from djangoplus import app_settings

class ModelInfoBase(object):
    class _Meta:
        model = None
        fields = ()
        exclude = ()
        show_if_none = False
        show_if_empty = False
        auto_urlize = True
        auto_linebreaks = True
        list_display_links = ()

    request = None

    def get_model_fields(self):
        ret = [f.name for f in self._meta.model._meta.fields \
                if f.name != 'id' and \
                (f.name in self._meta.fields or \
                not f.name in self._meta.exclude)\
                ]
        return ret

    def get_field(self, f_name):
        try:
            return [f for f in self._meta.model._meta.fields if f.name == f_name][0]
        except IndexError, e:
            return None

    def get_field_display_text(self, f_name):
        """Returns the display text for the informed field. This allows you
        declare a method like 'get_FIELD_display' to returns a customized
        display text."""

        if hasattr(self, 'get_%s_display'%f_name):
            ret = getattr(self, 'get_%s_display'%f_name)()
        else:
            f = self.get_field(f_name)

            if f:
                ret = f.verbose_name
            else:
                ret = f_name

        return ret[0] == ret[0].lower() and ret.capitalize() or ret

    def get_field_display_value(self, f_name, instance):
        field = self.get_field(f_name)
        f_value = None

        try:
            return getattr(self, 'get_%s_value'%f_name)(instance)
        except AttributeError, e:
            pass

        if isinstance(instance, models.Model):
            try:
                f_value = getattr(instance, f_name)
            except ObjectDoesNotExist:
                f_value = None
        elif type(instance) == types.DictType:
            f_value = instance.get(f_name, None)

        if f_value is None:
            return None

        if callable(f_value):
            return f_value()
        
        if isinstance(f_value, models.Model):
            if self._meta.auto_urlize and hasattr(f_value, 'get_absolute_url') and not f_name in self._meta.list_display_links:
                return '<a href="%s">%s</a>'%(f_value.get_absolute_url(), unicode(f_value))
            else:
                return unicode(f_value)

        if field:
            if field.choices:
                return dict(field.choices).get(f_value, None)

            if isinstance(field, models.BooleanField):
                return yesno(f_value)

            date_format, datetime_format, time_format = get_date_formats()

        if f_value:
            #if isinstance(field, models.DateTimeField):
            if isinstance(f_value, datetime):
                return dateformat.format(f_value, datetime_format)

            #if isinstance(field, models.TimeField):
            if isinstance(f_value, time):
                return dateformat.time_format(f_value, time_format)

            #if isinstance(field, models.DateField):
            if isinstance(f_value, date):
                return dateformat.format(f_value, date_format)

            if field and isinstance(field, models.TextField):
                if self._meta.auto_urlize: f_value = urlize(f_value)
                if self._meta.auto_linebreaks: f_value = linebreaksbr(f_value)

        return f_value

    def get_linkable_field_value(self, instance, f_name, f_value, force=False):
        url = ''

        if isinstance(instance, models.Model) and hasattr(instance, 'get_absolute_url'):
            url = instance.get_absolute_url()
        elif type(instance) == types.DictType and instance.has_key('get_absolute_url'):
            url = instance['get_absolute_url']

        if hasattr(instance, 'get_absolute_url') and ( force or f_name in self._meta.list_display_links ):
            return '<a href="%s">%s</a>' %(instance.get_absolute_url(), f_value)

        return f_value

    def as_string(self):
        raise NotImplemented

    def __unicode__(self):
        return self.as_string()

class ModelInfo(ModelInfoBase):
    """Automatic info creator for model classes, basing on objects"""
    class _Meta(ModelInfoBase._Meta):
        fieldsets = None
        row_template = '<tr><th>%s</th><td>%s</td></tr>'
        fieldset_title_template = '<h3>%s</h3>'
        show_fieldset_title = True

    def __init__(self, instance, *args, **kwargs):
        self.instance = instance

        if 'request' in kwargs:
            self.request = kwargs['request']

        self._meta = self.Meta()
        _meta = self._Meta()

        for attr in ('model','fields','exclude','fieldsets','row_template','show_if_none',
                'show_if_empty','auto_urlize','auto_linebreaks','list_display_links',
                'fieldset_title_template','show_fieldset_title'):
            if not hasattr(self._meta, attr):
                setattr(self._meta, attr, getattr(_meta, attr))

    def get_field_display_value(self, f_name):
        return super(ModelInfo, self).get_field_display_value(f_name, self.instance)

    def fieldset_render(self, s_name, s_fields):
        ret = []

        if s_name and self._meta.show_fieldset_title:
            ret.append(self._meta.row_template%('&nbsp;', self._meta.fieldset_title_template % s_name))
                
        for f_name in s_fields:
            f_display = self.get_field_display_text(f_name)

            f_value = self.get_field_display_value(f_name)
            f_value = self.get_linkable_field_value(self.instance, f_name, f_value)

            if (self._meta.show_if_none or f_value is not None) and\
               (self._meta.show_if_empty or f_value != ''):
                ret.append(self._meta.row_template%(f_display, f_value))

        return ret

    def __getitem__(self, num):
        fieldsets = self._meta.fieldsets or ((None, self._meta.fields or self.get_model_fields()),)

        s_name, s_fields = fieldsets[num]

        ret = self.fieldset_render(s_name, s_fields)

        return mark_safe(u'\n'.join(ret))

    def as_string(self):
        ret = []

        fieldsets = self._meta.fieldsets or ((None, self._meta.fields or self.get_model_fields()),)

        for s_name, s_fields in fieldsets:
            ret += self.fieldset_render(s_name, s_fields)

        return mark_safe(u'\n'.join(ret))

class ModelListItem(object):
    model_list = None
    _row_str = ''
    obj = None

    def __init__(self, model_list, row_str, obj):
        self.model_list = model_list
        self._row_str = row_str
        self.obj = obj

    def __unicode__(self):
        return self.row_str

    def get_row_str(self):
        return self._row_str
    row_str = property(get_row_str)

class ModelList(ModelInfoBase):
    """Automatic list creator for model classes, basing on QuerySets"""
    item_class = ModelListItem

    class _Meta(ModelInfoBase._Meta):
        td_template = '<td class="%(field_name)s">%(value)s</td>'
        th_template = '<th class="%(field_name)s">%(display)s</th>'
        tr_template = '<tr>%s</tr>'
        thead_template = '<thead><tr>%s</tr></thead>'
        tbody_template = '<tbody>%s</tbody>'
        icon_edit_template = '<a href="%(edit_url)s" title="Edit this"><img src="%(media_url)simg/admin/icon_changelink.gif" alt="Edit"/></a>'
        icon_delete_template = '<a href="%(delete_url)s" title="Delete this"><img src="%(media_url)simg/admin/icon_deletelink.gif" alt="Edit"/></a>'
        group_template = '<tr><td colspan="%(cols)s" class="group"><h3>%(display)s</h3></td></tr>'
        groups = []
        show_header = True
        show_summary = True
        summary_fields = None
        summary_td_template = '<td class="%(field_name)s">%(value)s</td>'
        summary_tr_template = '<tr>%(cells)s</tr>'

    def __init__(self, queryset, *args, **kwargs):
        self.queryset = queryset

        if 'request' in kwargs:
            self.request = kwargs['request']

        self._meta = self.Meta()
        _meta = self._Meta()

        for attr in ('model','fields','exclude','td_template','th_template','tr_template',
                'thead_template','tbody_template','show_if_none','auto_urlize',
                'auto_linebreaks','list_display_links','icon_edit_template',
                'icon_delete_template','group_template','groups','show_header',
                'summary_fields','summary_td_template','summary_tr_template','show_summary'):
            if not hasattr(self._meta, attr):
                setattr(self._meta, attr, getattr(_meta, attr))

        # Replace "%s" for "%(value)s" on td_template to accept old uses
        self._meta.td_template = self._meta.td_template.replace(r'%s', r'%(value)s')

    def render_buttons_header(self):
        """Renders the header cell for the column that will have util buttons"""
        if self._meta.icon_edit_template or self._meta.icon_delete_template:
            return '<th class="buttons">&nbsp;</th>'
        else:
            return ''

    def render_buttons_cell(self, instance, edit_url=None, delete_url=None, additional_code=''):
        """Renders the cell with util buttons for each row in the grid"""
        ret = []

        url = hasattr(instance, 'get_absolute_url') and getattr(instance, 'get_absolute_url')() or ''
        edit_url = edit_url or (url and url+'edit/' or '')
        delete_url = delete_url or (url and url+'delete/' or '')

        if self._meta.icon_edit_template and edit_url:
            ret.append(self._meta.icon_edit_template%{'edit_url':edit_url, 'media_url':settings.ADMIN_MEDIA_PREFIX})

        if self._meta.icon_delete_template and delete_url:
            ret.append(self._meta.icon_delete_template%{'delete_url':delete_url, 'media_url':settings.ADMIN_MEDIA_PREFIX})

        if ret:
            return '<td class="buttons">%s %s</td>'%(' '.join(ret), additional_code)
        else:
            return ''

    def header(self):
        """Renders a header with field labels on the top of the grid"""
        fields = self._meta.fields or self.get_model_fields()

        # THead
        thead = []

        for f_name in fields:
            f_display = self.get_field_display_text(f_name)
            f_display = f_display[0] == f_display[0].lower() and f_display.capitalize() or f_display

            thead.append(self._meta.th_template%{'field_name': f_name, 'display': f_display})

        return self._meta.thead_template %(''.join(thead) + self.render_buttons_header())

    def summary(self):
        """Renders a summary with aggregation in the end of the grid.
        
        Uses the meta attributes:
            
         * show_summary
         * summary_fields
         * summary_td_template
         * summary_tr_template
        """
        fields = self._meta.fields or self.get_model_fields()

        try:
            qs = self.queryset.all()
        except AttributeError:
            qs = self.queryset

        tsummary = []

        for f_name in fields:
            if f_name in self._meta.summary_fields:
                val = self._meta.summary_fields[f_name](qs)
            else:
                val = '&nbsp'
            
            if isinstance(val, decimal.Decimal):
                val = moneyformat(val, None, app_settings.THOUSANDS_SEPARATOR)

            tsummary.append(self._meta.summary_td_template%{
                'field_name': f_name,
                'value': val,
                })

        return self._meta.summary_tr_template%{
                'cells': ''.join(tsummary) + self.render_button_cell_summary(),
                }

    def render_button_cell_summary(self):
        return '<td>&nbsp;</td>'

    def get_columns_count(self):
        fields = self._meta.fields or self.get_model_fields()
        return len(fields)+1

    def rows(self):
        """Iterator with the list of rows in the grid"""
        fields = self._meta.fields or self.get_model_fields()

        tbody = []

        groups_values = {}
        if self._meta.groups:
            groups_values = groups_values.fromkeys(self._meta.groups)

        try:
            qs = self.queryset.all()

            if self._meta.groups:
                try:
                    qs = qs.order_by(*self._meta.groups)
                except AssertionError, e:
                    # Ignores the error if it doesn't get to order
                    if e.message != 'Cannot reorder a query once a slice has been taken.':
                        raise

        except AttributeError:
            qs = self.queryset

        for obj in qs:
            # Groups
            group_rows = []

            if self._meta.groups:
                for group in self._meta.groups:
                    field_value = self.get_field_display_value(group, obj)
                    if groups_values[group] != field_value:
                        groups_values[group] = field_value
                        group_row = self._meta.group_template%{'cols': self.get_columns_count(), 'display': field_value}
                        group_rows.append(group_row)

            # Single row
            row = self.render_single_object(obj)
            ret = self.item_class(self, '\n'.join(group_rows) + row, obj)

            yield ret

    def render_single_object(self, obj, tr_template=None, td_template=None):
        """Renders a single object. It is util internally or can be used for
        granulary customizations"""
        # Customized templates
        tr_template = tr_template or self._meta.tr_template
        td_template = td_template or self._meta.td_template

        fields = self._meta.fields or self.get_model_fields()
        row = []

        for i, f_name in enumerate(fields):
            f_value = self.get_field_display_value(f_name, obj)
            f_value = self.get_linkable_field_value(
                    obj, f_name, f_value,
                    force=not self._meta.list_display_links and i == 0,
                    )

            if isinstance(f_value, decimal.Decimal):
                f_value = moneyformat(f_value, None, app_settings.THOUSANDS_SEPARATOR)

            if self._meta.show_if_none or f_value is not None:
                row.append(td_template%{'field_name': f_name, 'value': f_value})
            else:
                row.append(td_template%{'field_name': f_name, 'value': '&nbsp;'})

        ret = tr_template%(''.join(row) + self.render_buttons_cell(obj))

        return ret

    def as_string(self):
        """Returns the whole string with the grid HTML code"""
        fields = self._meta.fields or self.get_model_fields()

        # THead
        if self._meta.show_header:
            thead = self.header()
        else:
            thead = ''

        # TBody
        tbody = [r.row_str for r in self.rows()]

        if tbody:
            tbody = self._meta.tbody_template % u'\n'.join(tbody)
            ret = u'\n'.join([thead, tbody])
        else:
            ret = u''

        # Summary
        if self._meta.show_summary and self._meta.summary_fields:
            ret += self.summary()

        return mark_safe(ret)




import copy

from django.forms.forms import BoundField
from django.db import models
from django.utils.safestring import mark_safe
from django import forms

class CrossTableForm(forms.Form):
    """Customized form class to be extended and used to let users inform
    fields in a cross reference table matrix.
    
    A cross table is a 3-D relationship in a mode classe where you can
    inform an X and an Y fields, and a field to be used as a cross
    between them."""

    class Meta: pass
    class CrossFields: pass
    class InfoFields: pass

    class _Meta:
        model = None
        x_model = None # Still to be checked and used TODO
        y_model = None # Still to be checked and used TODO
        x_field = None
        y_field = None
        cross_fields_template = 'col-%s-%s-'

    x_values = None
    y_values = None
    queryset = None

    def __init__(self, *args, **kwargs):
        # Meta class
        self.init_meta()

        # Cross and Info fields
        self.init_info_fields()
        self.init_cross_fields()

        # X and Y values lists
        self.x_values = kwargs.pop('x_values', [])
        self.y_values = kwargs.pop('y_values', [])

        # Queryset to base on
        self.queryset = kwargs.pop('queryset', None)

        # Create cross fields
        self.create_fields()

        super(CrossTableForm, self).__init__(*args, **kwargs)

        # Loads initial values from queryset
        if self.queryset:
            self.load_initial_from_queryset(self.queryset)

    def init_meta(self):
        """Set meta object from meta class"""
        self._meta = self.Meta()
        _meta = self._Meta()

        for attr in ('model','x_model','y_model','x_field','y_field','cross_fields_template',):
            if not hasattr(self._meta, attr):
                setattr(self._meta, attr, getattr(_meta, attr))

        # Find X relation field TODO
        # Find Y relation field TODO

    def init_info_fields(self):
        """Transform fields from InfoFields subclass into info_fields dictionary"""
        self.info_fields = [(k, getattr(self.InfoFields, k)) for k in dir(self.InfoFields)
            if isinstance(getattr(self.InfoFields, k), forms.Field)]

    def init_cross_fields(self):
        """Transform fields from CrossFields subclass into cross_fields dictionary"""
        self.cross_fields = [(k, getattr(self.CrossFields, k)) for k in dir(self.CrossFields)
            if isinstance(getattr(self.CrossFields, k), forms.Field)]

    def create_fields(self):
        """Create fields for Y values"""
        for x_value in self.x_values:
            for y_value in self.y_values:
                for name, field in self.cross_fields:
                    field_prefix = self._meta.cross_fields_template%(
                            self.get_x_value(x_value),
                            self.get_y_value(y_value),
                            )
                    self.base_fields[field_prefix + name] = copy.deepcopy(field)

    def get_x_value(self, value):
        """This method is to get a valid referene (and unique) value from a field
        value.
        
        This value is used to make the field names, then you can't use rich
        formatted values here. Only simple values with no spaces, special chars,
        symbols, etc.
        
        You can extend it to customize your form."""

        if isinstance(value, models.Model):
            return value.pk

        return value

    def get_y_value(self, value):
        """See 'get_x_value' docs. It's the same, but used for Y field values."""

        if isinstance(value, models.Model):
            return value.pk

        return value

    @property
    def columns(self):
        """Returns a generator with column headers"""
        column_labels = [field[1].label for field in self.info_fields] + [unicode(value) for value in self.y_values]

        for label in column_labels:
            yield u'<th>%s</th>'%unicode(label)

    @property
    def rows(self):
        """Returns a generator with table rows"""
        for x_value in self.x_values:
            tds = []

            # Info fields
            for name, field in self.info_fields:
                field = BoundField(self, field, name)
                tds.append(u'<td>%s</td>'%unicode(field))

            # Cross fields
            for y_value in self.y_values:
                field_prefix = self._meta.cross_fields_template%(
                        self.get_x_value(x_value),
                        self.get_y_value(y_value),
                        )

                for field_name, field in self.fields.items():
                    if field_name.startswith(field_prefix):
                        errors = '' # TODO
                        bfield = BoundField(self, field, field_name)

                        tds.append(u'<td>%s %s</td>'%(errors, unicode(bfield)))

            tds = u''.join(tds)

            yield u'<tr><th>%s</th>%s</tr>'%(x_value, tds)

    def as_table(self):
        """Returns form renderized to HTML TABLE format"""

        ths = u''.join([th for th in self.columns])
        ths = u'<tr><th>&nbsp;</th>%s</tr>'%ths

        trs = u'\n'.join([tr for tr in self.rows])

        return mark_safe('\n'.join([ths, trs]))

    def __unicode__(self):
        return self.as_table()

    def get_cross_object(self, queryset, x_value, y_value):
        """Find object for the cross between X and Y relationship"""

        filters = {
                self._meta.x_field: x_value,
                self._meta.y_field: y_value,
                }

        try:
            return queryset.get(**filters)
        except queryset.model.DoesNotExist:
            return None

    def get_field_value_from_queryset(self, queryset, x_value, y_value, name, field):
        """Used be initials loader from queryset method to get the value for just a
        field crossed by X and Y values"""
        
        obj = self.get_cross_object(queryset, x_value, y_value)

        return getattr(obj, name)

    def load_initial_from_queryset(self, queryset):
        """Loads intial values from queryset, using meta defined fields and their
        values to find the cross fields values"""

        # Loads initials for cross fields
        for x_value in self.x_values:
            for y_value in self.y_values:
                field_prefix = self._meta.cross_fields_template%(x_value, y_value)
                fields = [f for f in self.cross_fields if f[0].startswith(field_prefix)]

                for name, field in fields:
                    field.initial = self.get_field_value_from_queryset(queryset, x_value, y_value, name, field)

        # Loads initials for info fields TODO

    def create_cross_object(self, queryset, x_value, y_value, extra_fields=None):
        """Returns an object with X and Y fields informed, and additional extra
        fields values if they are informed, also, to be saved as a new object."""

        obj = self._meta.model()
        setattr(obj, self._meta.x_field, x_value)
        setattr(obj, self._meta.y_field, y_value)

        # Sets additional fields
        extra_fields = extra_fields or {}
        for k, v in extra_fields.items():
            setattr(obj, k, v)

        return obj

    def save_cross(self, x_value, y_value):
        """Saves all values to the database"""

        # Get object to save into
        obj = self.get_cross_object(
                self.queryset,
                x_value,
                y_value,
                ) or self.create_cross_object(
                        self.queryset,
                        x_value,
                        y_value,
                        )

        # Sets cross fields values
        for field in self.cross_fields:
            field_prefix = self._meta.cross_fields_template%(
                self.get_x_value(x_value),
                self.get_y_value(y_value),
                )
            field_name = field_prefix + field[0]

            setattr(obj, field[0], self.cleaned_data[field_name])

        obj.save()

    def save(self):
        """Saves all values to the database"""
        for x_value in self.x_values:
            for y_value in self.y_values:
                self.save_cross(x_value, y_value)

class CrossTableManager(models.Manager):
    """Manager to add support to cross table functions with 3 fields from a model
    class.
    
    Cross tables are matrix where you have an X field for rows, Y field for
    columns and a VALUE field for coordinates between X for Y to relate them."""

    def get_query_set(self):
        return CrossTableQuerySet(self.model)

    def crosstable(self, x_field, y_field, cross_fields, x_values=None, y_values=None):
        return self.get_query_set().crosstable(x_field, y_field, cross_fields, x_values,
                y_values)

class CrossTableQuerySet(models.query.QuerySet):
    """QuerySet class to add support to cross table functions with 3 fields from a
    model class. This should be used together with CrossTableManager"""

    def crosstable(self, x_field, y_field, cross_fields, x_values=None, y_values=None):
        """Returns a matrix with crosstable"""
        qs = self

        # Gets X values
        x_values = x_values or sets.Set([getattr(obj, x_field) for obj in qs])
        x_values = [i for i in x_values]

        # Gets Y values
        y_values = y_values or sets.Set([getattr(obj, y_field) for obj in qs])
        y_values = [i for i in y_values]

        # Make easier use of cross fields attribute
        cross_fields = isinstance(cross_fields, (list,tuple)) and cross_fields or [cross_fields]

        ret = [[None] + y_values]

        # Values getter
        def get_xy_value(x_value, y_value):
            for obj in qs:
                if getattr(obj, x_field) == x_value and getattr(obj, y_field) == y_value:
                    ret = [getattr(obj, field) for field in cross_fields]
                    ret = len(ret) > 1 and ret or ret[0]
                    return ret

            return None

        for x_value in x_values:
            ret.append([x_value] + [get_xy_value(x_value, y_value) for y_value in y_values])

        return ret




import base64, types, urllib
from datetime import datetime, timedelta

from django import template
from django.template import Library, Node
from django.utils.translation import ugettext as _
from django.utils.safestring import SafeUnicode, mark_safe
from django.conf import settings
from django.template.defaultfilters import slugify
from django.core.cache import cache
from django.contrib.contenttypes.models import ContentType

from djangoplus.utils import path_to_object, split1000, get_admin_url
from djangoplus import app_settings

register = Library()

"""
Boolean general utilities
"""

@register.filter
def multiple_of(value,arg):
    return value % arg == 0

@register.filter
def divide_by(value,arg):
    return value / arg

@register.filter
def in_list(value,arg):
    if not value:
        return False

    if type(arg) in (types.StringType, SafeUnicode):
        arg = arg.split(',')
    
    return value in arg

@register.filter
def is_equal(value,arg):
    return value == arg

@register.filter
def as_string(value):
    # Returns unicode convertion of value
    return unicode(value)

@register.filter
def as_integer(value):
    """Converts a number to integer"""
    return int(value)

@register.filter
def is_not_equal(value,arg):
    return value != arg

@register.filter
def is_lt(value,arg):
    return int(value) < int(arg)

@register.filter
def is_lte(value,arg):
    return int(value) <= int(arg)

@register.filter
def is_gt(value,arg):
    return int(value) > int(arg)

@register.filter
def is_gte(value,arg):
    return int(value) >= int(arg)

@register.filter
def replace(value,arg):
    arg, arg2 = arg.split(':')
    return value.replace(arg, arg2)

"""
Date/time util template filters
"""

@register.filter
def is_day_of(value,arg):
    return (arg and value) and arg.day == int(value)

@register.filter
def is_month_of(value,arg):
    return (arg and value) and arg.month == int(value)

@register.filter
def is_year_of(value,arg):
    return (arg and value) and arg.year == int(value)

@register.filter
def is_hour_of(value,arg):
    return (arg and value) and arg.hour == int(value)

@register.filter
def is_minute_of(value,arg):
    return (arg and value) and arg.minute == int(value)

@register.filter
def dec_year(value,arg):
    delta = type(arg) != types.IntType and int(arg) or arg
    return value - timedelta(365 * delta)

@register.filter
def dec_month(value,arg):
    delta = type(arg) != types.IntType and int(arg) or arg
    return value - timedelta(30 * delta)

@register.filter
def inc_year(value,arg):
    delta = type(arg) != types.IntType and int(arg) or arg
    return value + timedelta(365 * delta)

@register.filter
def inc_month(value,arg):
    delta = type(arg) != types.IntType and int(arg) or arg
    return value + timedelta(30 * delta)

@register.filter
def list_as_text(value, field=None):
    if field:
        def get_value(obj, attr):
            attr = getattr(obj, attr)

            if type(attr) == types.MethodType:
                return attr()

            return attr or ''

        return ', '.join([get_value(i, field) for i in value])
    else:
        return ', '.join([unicode(i) for i in value])

@register.filter
def list_as_links(value):
    return ', '.join(['<a href="%s">%s</a>'%(i.get_absolute_url(), unicode(i)) for i in value])

"""
reStructuredText
"""

@register.filter
def rest(value, arg):
    from docutils.core import publish_parts

    arg = arg or 'html'
    parts = publish_parts(value, writer_name=arg)
    return parts['html_body']

"""
Strings
"""

@register.filter
def startswith(value,arg):
    return value.startswith(arg)

@register.filter
def endswith(value,arg):
    return value.endswith(arg)

@register.filter
def replace(value,arg):
    args = arg.split(',')
    return value.replace(args[0], args[1])

@register.filter
def is_lower(value):
    return value == value.lower()

@register.filter
def is_upper(value):
    return value == value.upper()

@register.filter
def is_capitalized(value):
    return value == value.capitalize()

"""
Miscelanea
"""

def multifind(s, q):
    # Returns a list of positions from a string in other one - used by 'highlight'
    ret = []
    sl = s.lower()
    q = q.lower()
    i = 0

    while i < len(sl):
        aux = sl[i:]
        if aux.find(q) == 0:
            ret.append(i)
            i += len(q)
        else:
            i += 1

    return ret

@register.filter
def highlight(s, q):
    pos = multifind(s, q)

    if not pos: return s

    anterior = 0
    ret = ''
    
    for p in pos:
        if anterior:
            ret += s[anterior+len(q):p]
        else:
            ret += s[anterior:p]

        ret += '<span class="highlight">' + s[p:p+len(q)] + '</span>'
        anterior = p

    ret += s[p+len(q):]

    return ret

@register.filter_function
def attr(obj, arg1):
    """Sets an HTML attribute to a form widget"""
    att, value = arg1.split("=")
    obj.field.widget.attrs[att] = value
    return obj

@register.filter_function
def order_by(queryset, args):
    """Returns a queryset filtered ordered by the field informed"""
    args = [x.strip() for x in args.split(',')]
    return queryset.order_by(*args)

@register.filter_function
def make_cloud(object_list, count_attribute, steps=5):
    """Make a cloud on the given list, using counter attribute informed"""
    def get_count(obj):
        if not hasattr(obj, '__cloud_counter'):
            cur = obj
            for attr in count_attribute.split('.'):
                cur = getattr(cur, attr)

            if type(cur) == types.MethodType:
                setattr(obj, '__cloud_counter', cur())
            else:
                setattr(obj, '__cloud_counter', cur)

        return obj.__cloud_counter

    def get_font_size(obj, steps_counts):
        for i, step in enumerate(steps_counts):
            if get_count(obj) < step:
                return i

        return steps

    def steps_between_to_numbers(first, second):
        div = float(second - first) / (steps-1)
        return [first+int(round(div * i)) for i in range(steps)]

    min_count = min([get_count(obj) for obj in object_list])
    max_count = max([get_count(obj) for obj in object_list])
    steps_counts = steps_between_to_numbers(min_count, max_count)

    return [(obj, get_font_size(obj, steps_counts)) for obj in object_list]

@register.filter_function
def moneyformat(value, decimal_places=2, thousands_separator=''):
    """Returns a float value in monetary format, using MONETARY_LOCALE
    setting and locale functions"""
    decimal_places = decimal_places is None and 2 or decimal_places
    thousands_separator = thousands_separator or app_settings.THOUSANDS_SEPARATOR

    import locale

    if app_settings.MONETARY_LOCALE:
        locale.setlocale(locale.LC_NUMERIC, app_settings.MONETARY_LOCALE)

    format = r"%0.0"+str(decimal_places)+'f'

    value = value or 0.0

    ret = locale.format(format, float(value), grouping=True)

    if thousands_separator:
        dec_sep = thousands_separator == ',' and '.' or ','
        parts = ret.split(dec_sep)
        ret = split1000(parts[0].replace(thousands_separator, ''), thousands_separator)

        if len(parts) > 1:
            ret += dec_sep + parts[1]

    return ret

@register.filter_function
def has_module_perm(user, app_label):
    """Checks if that user has some permission on that application and returns
    True or False for this."""
    if not user or not user.is_authenticated:
        return False

    if user.is_superuser:
        return True

    return user.has_module_perms(app_label)

@register.filter
def admin_url(obj):
    return get_admin_url(obj)

"""
TAMPLATE TAGS
"""

class ProtectAntiRobotsNode(Node):
    phrase = ''
    nodelist = ''

    def __init__(self, nodelist, phrase=None):
        super(ProtectAntiRobotsNode, self).__init__()

        self.nodelist = nodelist
        self.phrase = phrase or _('Let us know if you are human')

    def __repr__(self):
        return "<ProtectAntiRobotsNode>"

    def render(self, context):
        request = context['request']
        path = slugify(request.get_full_path())
        if 'protectantirobots_sec_'+path in request.COOKIES and \
           'protectantirobots_key_'+path in request.COOKIES:
            if request.COOKIES['protectantirobots_sec_'+path] == base64.b64decode(request.COOKIES['protectantirobots_key_'+path]):
                output = self.nodelist.render(context)
                return output
        
        sec = base64.b64encode(datetime.now().strftime("%H%m%S"))
        return '<a href="%s">%s</a>' %(self.get_url(sec, path), self.phrase)

    def get_url(self, sec, path):
        return "/protectantirobots/?k=%s&path=%s"%(sec, path)

def do_protectantirobots(parser, token):
    nodelist = parser.parse(('endprotectantirobots',))
    parser.delete_first_token()

    parts = token.split_contents()
    phrase = len(parts) > 1 and parts[1][1:-1] or None

    return ProtectAntiRobotsNode(nodelist, phrase)
do_protectantirobots= register.tag('protectantirobots', do_protectantirobots)

"""
{% dynamic_template [group] "name" %}
"""

from djangoplus.models import DynamicTemplate

class DynamicTemplateRender(template.Node):
    slug = None
    is_group = False

    def __init__(self, slug, is_group=False):
        self.slug = slug
        self.is_group = is_group

    def render(self, context):
        ret = ''

        if self.is_group:
            templates = DynamicTemplate.objects.filter(group=self.slug)
        else:
            templates = DynamicTemplate.objects.filter(slug=self.slug)

        for tpl in templates:
            ret += tpl.render(context)

        return ret

def do_dynamic_template(parser, token):
    try:
        parts = token.split_contents()
        tag_name = parts[0]
        slug = parts[-1]

        if len(parts) > 3:
            raise ValueError('Many arguments')
        elif len(parts) > 2 and parts[1] == 'group':
            is_group = True
        else:
            is_group = False
    except ValueError, e:
        raise template.TemplateSyntaxError, "%s requires 1 or 2 arguments" \
                % token.contents.split()[0]

    return DynamicTemplateRender(slugify(slug), is_group)

register.tag('dynamic_template', do_dynamic_template)

class RaiseRender(template.Node):
    vobj = None
    vadditional = None

    def __init__(self, vobj, vadditional=None):
        self.vobj = template.Variable(vobj)
        self.vadditional = vadditional

    def render(self, context):
        obj = self.vobj.resolve(context)

        if self.vadditional:
            obj = getattr(obj, self.vadditional)

        raise Exception(obj.__dict__)

def do_raise(parser, token):
    try:
        parts = token.split_contents()
        tag_name = parts[0]
        vobj = parts[1]
        vadditional = len(parts) == 3 and parts[2] or None
    except ValueError, e:
        raise template.TemplateSyntaxError, "%s requires 1 or 2 arguments" \
                % token.contents.split()[0]

    return RaiseRender(vobj, vadditional)

register.tag('raise', do_raise)

class ChangeGetParamsNode(template.Node):
    param = None
    new_value = None

    def __init__(self, param, new_value):
        self.param = template.Variable(param)
        self.new_value = template.Variable(new_value)

    def render(self, context):
        from urllib import urlencode

        # Resolve variables
        param = self.param.resolve(context)
        new_value = self.new_value.resolve(context)

        # Change current params
        cur_params = dict([(k, v.encode('utf-8')) for k,v in context['request'].GET.items()])
        cur_params[param] = new_value

        return urlencode(cur_params)

def do_change_get_params(parser, token):
    try:
        parts = token.split_contents()
        param = parts[1]
        new_value = parts[2]
    except ValueError:
        raise template.TemplateSyntaxError, "%s requires 2 arguments"\
                % token.split_contents()[0]

    return ChangeGetParamsNode(param, new_value)

register.tag('change_get_params', do_change_get_params)

class RemoveFromGetParamsNode(template.Node):
    param = None

    def __init__(self, param):
        self.param = template.Variable(param)

    def render(self, context):
        from urllib import urlencode
        # Resolve variables
        param = self.param.resolve(context)

        # Change current params
        cur_params = dict([(k, v.encode('utf-8')) for k,v in context['request'].GET.items() if k != param])

        return urlencode(cur_params)

def do_remove_from_get_params(parser, token):
    try:
        parts = token.split_contents()
        param = parts[1]
    except ValueError:
        raise template.TemplateSyntaxError, "%s requires 1 argument"\
                % token.split_contents()[0]

    return RemoveFromGetParamsNode(param)

register.tag('remove_from_get_params', do_remove_from_get_params)

# Model info

class ModelInfoForObjectNode(template.Node):
    class_path = None
    obj = None
    as_part = None
    as_varname = None

    def __init__(self, class_path, obj, as_part=None, as_varname=None):
        self.class_path = template.Variable(class_path)
        self.obj = template.Variable(obj)
        self.as_part = as_part
        self.as_varname = as_varname

    def render(self, context):
        class_path = self.class_path.resolve(context)
        obj = self.obj.resolve(context)

        cls = path_to_object(class_path)
        model_info = cls(obj, request=context.get('request', None))

        if not self.as_part:
            return model_info

        context[self.as_varname] = model_info
        return ''

def do_model_info_for_object(parser, token):
    """Returns the model informations for an object: its fields and their values"""

    try:
        parts = token.split_contents()
    except ValueError:
        raise template.TemplateSyntaxError, "You must inform class path and object"

    return ModelInfoForObjectNode(*parts[1:])

register.tag('model_info_for_object', do_model_info_for_object)


class ModelInfoForListNode(template.Node):
    class_path = None
    list_obj = None
    as_varname = None
    piece = None

    def __init__(self, class_path, list_obj, as_varname=None, piece=None):
        self.class_path = template.Variable(class_path)
        self.list_obj = template.Variable(list_obj)
        self.as_varname = as_varname
        self.piece = piece

    def render(self, context):
        class_path = self.class_path.resolve(context)
        list_obj = self.list_obj.resolve(context)

        cls = path_to_object(class_path)
        model_list = cls(list_obj, request=context.get('request', None))

        if self.piece:
            model_list = getattr(model_list, self.piece, None)

            if callable(model_list):
                model_list = model_list()

        if not self.as_varname:
            return model_list

        context[self.as_varname] = model_list
        return ''

def do_model_info_for_list(parser, token):
    """Returns the model informations for a list: a table with a list of objects"""

    try:
        parts = token.split_contents()

        class_path = parts[1]
        list_obj = parts[2]

        if 'as' in parts:
            as_varname = parts[parts.index('as')+1]
        else:
            as_varname = None
            
        if 'only' in parts:
            piece = parts[parts.index('only')+1]
        else:
            piece = None

    except ValueError:
        raise template.TemplateSyntaxError, "You must inform class path and list object"

    return ModelInfoForListNode(class_path, list_obj, as_varname, piece)

register.tag('model_info_for_list', do_model_info_for_list)

class ModelInfoFields(template.Node):
    class_path = None
    as_varname = None

    def __init__(self, class_path, as_varname=None):
        self.class_path = template.Variable(class_path)
        self.as_varname = as_varname

    def render(self, context):
        class_path = self.class_path.resolve(context)

        cls = path_to_object(class_path)
        m_obj = cls(None, request=context.get('request', None))
        fields = [(f, m_obj.get_field_display_text(f)) for f in m_obj._meta.fields]

        if not self.as_varname:
            return fields

        context[self.as_varname] = fields
        return ''

def do_model_info_fields(parser, token):
    """Returns the fields list on a model info or model object class"""

    try:
        parts = token.split_contents()

        class_path = parts[1]

        if 'as' in parts:
            as_varname = parts[parts.index('as')+1]
        else:
            as_varname = None
    except ValueError:
        raise template.TemplateSyntaxError, "You must inform class path, 'as' and the output variable name"

    return ModelInfoFields(class_path, as_varname)

register.tag('model_info_fields', do_model_info_fields)

class IncludeFromURLRender(template.Node):
    vurl = None
    vcache = None

    def __init__(self, vurl, vcache=None):
        self.vurl = template.Variable(vurl)
        self.vcache = vcache

    def render(self, context):
        url = self.vurl.resolve(context)
        url_key = 'includefromurl_'+slugify(url)

        # Load from cache?
        if self.vcache:
            cont = cache.get(url_key, '')
        else:
            cont = ''

        # If there is no content from cache, get from URL
        if not cont:
            fp = urllib.urlopen(url)
            cont = fp.read()
            fp.close()

        # Save on cache
        if self.vcache:
            cache.set(url_key, cont, int(self.vcache))

        return mark_safe(cont)

def do_includefromurl(parser, token):
    try:
        parts = token.split_contents()
        vurl = parts[1]
        vcache = len(parts) == 3 and parts[2] or None
    except ValueError, e:
        raise template.TemplateSyntaxError, "%s requires 1 or 2 arguments" \
                % token.contents.split()[0]

    return IncludeFromURLRender(vurl, vcache)

register.tag('includefromurl', do_includefromurl)

# Template tag 'checar_permissao'

class ContentTypeForModelRender(template.Node):
    vpath = None

    def __init__(self, vpath):
        self.vpath = template.Variable(vpath)

    def render(self, context):
        path = self.vpath.resolve(context).split('.')

        ctype = ContentType.objects.get(app_label=path[0], model=path[1])
        
        return ctype.pk

def do_content_type_for_model(parser, token):
    parts = token.split_contents()
    vpath = parts[1]

    return ContentTypeForModelRender(vpath)

register.tag('content_type_for_model', do_content_type_for_model)

# Ajax FK Widget

class AjaxFKWidgetCellsRender(template.Node):
    vobj = None
    vfields = None
    var_result = None

    def __init__(self, vobj, vfields, var_name):
        self.vobj = template.Variable(vobj)
        self.vfields = template.Variable(vfields)
        self.var_result = var_name

    def render(self, context):
        obj = self.vobj.resolve(context)
        list_display = self.vfields.resolve(context)

        context[self.var_result] = [{'name': f, 'value': getattr(obj, f, None)} for f in list_display]

        return ''

def do_ajaxfkwidget_cells(parser, token):
    try:
        parts = token.split_contents()
        vobj, vfields, var_name = parts[1:]
    except IndexError, e:
        raise template.TemplateSyntaxError, "%s requires 3 arguments" \
                % token.contents.split()[0]

    return AjaxFKWidgetCellsRender(vobj, vfields, var_name)

register.tag('ajaxfkwidget_cells', do_ajaxfkwidget_cells)

@register.filter
def ajaxfkwidget_url(obj, func):
    return func(obj)




from django.shortcuts import render_to_response
from django.core.mail import EmailMessage, SMTPConnection
from django.conf import settings
from django.template import Context
from django.template.loader import get_template
from django.http import HttpResponse

def render_to_json(obj):
    response = HttpResponse(obj)
    response['mimetype'] = "text/javascript"
    response['Pragma'] = "no cache"
    response['Cache-Control'] = "no-cache, must-revalidate"

    return response

def render_to_mail(template_path, params, subject, from_email=None, recipient_list=[], auth_user=None, auth_password=None, fail_silently=False, content_subtype=None, attachments=[], **kwargs): 
    """ 
    Sends a e-mail message with content parsed from a template. The syntax is  
    the same of render_to_response function. 
    Returns then boolean of mail sending, using core.mail.send_mail function. 
    """ 
    content_subtype = content_subtype or getattr(settings, 'DEFAULT_EMAIL_CONTENT_SUBTYPE', 'html') 
    from_email = from_email or getattr(settings, 'DEFAULT_FROM_EMAIL', None) 
    
    if not recipient_list:
        return False 

    # Loads template for mail content 
    t = get_template(template_path) 
    
    # Render content 
    message = t.render(Context(params)) 

    # SMTP connection
    connection = SMTPConnection(username=auth_user, password=auth_password, fail_silently=fail_silently)

    # Email object 
    email_obj = EmailMessage(subject, message, from_email, [r.strip() for r in recipient_list], connection=connection)
    
    email_obj.content_subtype = content_subtype 
    
    if attachments: 
        for att in attachments:
            email_obj.attach_file(att) 
    
    return email_obj.send() 




from django import forms
from django.utils.safestring import mark_safe

class ModelRawIdWidget(forms.TextInput):
    """
    A Widget for displaying ForeignKeys in the "raw_id" interface rather than
    in a <select> box.
    """
    def __init__(self, model, limit_choices_to=None, attrs=None):
        self.model = model
        self.limit_choices_to = limit_choices_to
        super(ModelRawIdWidget, self).__init__(attrs)

    def render(self, name, value, attrs=None):
        from django.conf import settings
        related_url = '/admin/%s/%s/' % (self.model._meta.app_label, self.model._meta.object_name.lower())
        if self.limit_choices_to:
            url = '?' + '&amp;'.join(['%s=%s' % (k, v) for k, v in self.limit_choices_to.items()])
        else:
            url = ''
        if not attrs.has_key('class'):
          attrs['class'] = 'vForeignKeyRawIdAdminField' # The JavaScript looks for this hook.
        output = [super(ModelRawIdWidget, self).render(name, value, attrs)]
        # TODO: "id_" is hard-coded here. This should instead use the correct
        # API to determine the ID dynamically.
        output.append('<a href="%s%s" class="related-lookup" id="lookup_id_%s" onclick="return showRelatedObjectLookupPopup(this);"> ' % \
            (related_url, url, name))
        output.append('<img src="%simg/admin/selector-search.gif" width="16" height="16" alt="Lookup" /></a>' % settings.ADMIN_MEDIA_PREFIX)
        if value:
            output.append(self.label_for_value(value))
        return mark_safe(u''.join(output))
    
    def label_for_value(self, value):
        return '&nbsp;<strong>%s</strong>' % \
            truncate_words(self.model.objects.get(pk=value), 14)




# django-djangoplus setup
# First version of this file done by Guilherme Semente
# Some things was copied from Django's setup.py
from distutils.command.install import INSTALL_SCHEMES
import os, sys

# Downloads setuptools if not find it before try to import
import ez_setup
ez_setup.use_setuptools()

from setuptools import setup
from djangoplus import get_version

# Tell distutils to put the data_files in platform-specific installation
# locations. See here for an explanation:
# http://groups.google.com/group/comp.lang.python/browse_thread/thread/35ec7b2fed36eaec/2105ee4d9e8042cb
for scheme in INSTALL_SCHEMES.values():
    scheme['data'] = scheme['purelib']

data_files = []

for dirpath, dirnames, filenames in os.walk('djangoplus'):
    # Ignore dirnames that start with '.'
    for i, dirname in enumerate(dirnames):
        if dirname.startswith('.'): del dirnames[i]
    if filenames:
        data_files.append([dirpath, [os.path.join(dirpath, f) for f in filenames]])

# Small hack for working with bdist_wininst.
# See http://mail.python.org/pipermail/distutils-sig/2004-August/004134.html
if len(sys.argv) > 1 and sys.argv[1] == 'bdist_wininst':
    for file_info in data_files:
        file_info[0] = '\\PURELIB\\%s' % file_info[0]

setup(
    name = 'django-plus',
    version = get_version(),
    description = 'Django utilities library',
    long_description = 'django-plus is a library containing a coupple of utilities for Django developers.',
    author = 'Marinho Brandao',
    author_email = 'marinho@gmail.com',
    url = 'http://django-plus.googlecode.com',
    license = 'GNU Lesser General Public License (LGPL)',
    packages = ['djangoplus', 'djangoplus.templatetags',
        'djangoplus.fieldtypes','djangoplus.forms','djangoplus.middleware',
        'djangoplus.shortcuts','djangoplus.templates',
        'djangoplus.templates.admin','djangoplus.templates.admin.djangoplus',
        'djangoplus.templates.admin.djangoplus.dynamictemplate',
        'djangoplus.templates.djangoplus','djangoplus.tests',
        'djangoplus.utils','djangoplus.views','djangoplus.widgets',],
    data_files = data_files,
)




import datetime
from django.db import models
from django.db.models import signals
from django.conf import settings
from django.utils import simplejson as json
from django.dispatch import dispatcher

"""
JSONField: http://www.djangosnippets.org/snippets/377/
"""

class JSONEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, datetime.datetime):
            return obj.strftime('%Y-%m-%d %H:%M:%S')
        elif isinstance(obj, datetime.date):
            return obj.strftime('%Y-%m-%d')
        elif isinstance(obj, datetime.time):
            return obj.strftime('%H:%M:%S')
        return json.JSONEncoder.default(self, obj)
        
def dumps(data):
    return JSONEncoder().encode(data)
    
def loads(str):
    return json.loads(str, encoding=settings.DEFAULT_CHARSET)
    
class JSONField(models.TextField):
    def db_type(self):
        return 'text'
        
    def pre_save(self, model_instance, add):
        value = getattr(model_instance, self.attname, None)
        return dumps(value)
    
    def contribute_to_class(self, cls, name):
        super(JSONField, self).contribute_to_class(cls, name)

        try:
            signals.post_init.connect(self.post_init, sender=cls)
        except:
            dispatcher.connect(self.post_init, signal=signals.post_init, sender=cls)
        
        def get_json(model_instance):
            return dumps(getattr(model_instance, self.attname, None))
        setattr(cls, 'get_%s_json' % self.name, get_json)
    
        def set_json(model_instance, json):
            return setattr(model_instance, self.attname, loads(json))
        setattr(cls, 'set_%s_json' % self.name, set_json)
    
    def post_init(self, instance=None, **kwargs):
        value = self.value_from_object(instance)
        if (value):
            setattr(instance, self.attname, loads(value))
        else:
            setattr(instance, self.attname, None)

